import { hostEmitter, createCommands, BaseError, EProject } from '@jolibox/common';
import { trackError } from './utils/event-tracker';
document.addEventListener('DOMContentLoaded', () => hostEmitter.emit('onDocumentReady', timeline.startTime));

const timelineFactory = () => {
  let _beginLoad = 0;
  return {
    set startTime(time: number) {
      _beginLoad = time;
    },
    get startTime(): number {
      return _beginLoad;
    }
  };
};

export const timeline = timelineFactory();

const commands = createCommands();

const rejectHandler = (event: PromiseRejectionEvent | ErrorEvent | BaseError) => {
  commands
    .executeCommand('ReportSDK.traceSystem', {
      event: 'globalJsError',
      info: {
        err: JSON.stringify(event)
      }
    })
    .catch((e) => {
      // 兜底上报
      console.error(`Fallback report: ${e} ${JSON.stringify(event)}`);
      trackError({
        err: JSON.stringify({
          message: e.message,
          source: e.source,
          lineno: e.lineno,
          colno: e.colno,
          error: e.error
        })
      });
    });
};

export const reportError = rejectHandler;

window.addEventListener('unhandledrejection', rejectHandler.bind(this));
window.addEventListener('error', rejectHandler.bind(this));
window.onerror = (message, source, lineno, colno, error) => {
  commands
    .executeCommand('ReportSDK.traceSystem', {
      event: 'globalJsError',
      info: {
        err: JSON.stringify({
          message,
          source,
          lineno,
          colno,
          error
        })
      }
    })
    .catch((e) => {
      // 兜底上报
      trackError({
        err: JSON.stringify({
          message,
          source,
          lineno,
          colno,
          error
        })
      });
    });
};
