import { Env, Lifecycle } from '@jolibox/types';
import { BaseSDK, BaseSDKEventMap } from './sdk';

const LIFECYCLE_ON_READY = 'LifecycleSDK.onReady';
interface LifecycleSDKEventMap extends BaseSDKEventMap {
  [LIFECYCLE_ON_READY]: Env['hostUserInfo'] | undefined;
}

export class LifecycleSDK extends BaseSDK<LifecycleSDKEventMap> implements Lifecycle {
  constructor() {
    super();
  }

  onReady(callback: (info?: Env['hostUserInfo']) => void) {
    const wrappedOnReady = (info?: Env['hostUserInfo']) => {
      callback.call(this, info);
      this.triggerEvent(LIFECYCLE_ON_READY, info);
    };
    this.commands.executeCommand('LifecycleSDK.onReady', wrappedOnReady.bind(this));
  }

  exit(params: { onBeforeExit: () => void }) {
    const errMsg = this.canIUseIfThrow('lifeCycle.exit');
    if (errMsg) {
      return errMsg;
    }
    this.commands.executeCommand('LifecycleSDK.exit', params.onBeforeExit);
  }

  onJoliboxHide(params: () => void) {
    this.commands.executeCommand('LifecycleSDK.onJoliboxHide', params.bind(this));
  }
  onJoliboxShow(params: () => void) {
    this.commands.executeCommand('LifecycleSDK.onJoliboxShow', params.bind(this));
  }
}
